
var p4200 = null;
var socket;
var pressureUnits = "PSI" // default
var flowUnits = "SCFD"
var tempUnits = "&degF" 

Number.prototype.pad = function (size) {
	var s = String(this);
	while (s.length < (size || 2)) { s = "0" + s; }
	return s;
}
/**
 * Open a new WebSocket connection using the given parameters
 */
function setupWebSocket(webSocket)
{
	try {
		webSocket.onopen = function (openEvent) {
			console.log("WebSocket OPEN: " + JSON.stringify(openEvent, null, 4));

		};
		webSocket.onclose = function (closeEvent) {
			console.log("WebSocket CLOSE: " + JSON.stringify(closeEvent, null, 4));
		};
		webSocket.onerror = function (errorEvent) {
			console.log("WebSocket ERROR: " + JSON.stringify(errorEvent, null, 4));
		};
		webSocket.onmessage = function (messageEvent) {
			var wsMsg = messageEvent.data;
			//console.log("WebSocket MESSAGE: " + wsMsg);
			p4200 = JSON.parse(wsMsg);
			updateUnits();
			updateAlarmStatus();
			updateTemps();
			updateSystem1();
			updatePressures();
			updateDevice();
			updateLockIcon();
			


		};
		socket = webSocket;
	} catch (exception) {
		console.error(exception);
	}
}

function updateUnits()
{
	
	if(p4200.sensors.metricUnitsEnabled == true)
	{
		pressureUnits = "KPA"
		flowUnits = "SCMD"
		tempUnits = "&degC" 
	}
	else
	{
		pressureUnits = "PSI"
		flowUnits = "SCFD"
		tempUnits = "&degF" 
	}
}
function updateDevice() {
	var model = document.getElementById('model_info');
	var status = document.getElementById('dryer_status');
	model.rows[0].cells[1].innerHTML = p4200.modelNumber;
	model.rows[1].cells[1].innerHTML = p4200.serialNumber;
	model.rows[2].cells[1].innerHTML = p4200.deviceID;
	var dryerStatus = p4200.status;
	if (dryerStatus == "ONLINE") {
		status.rows[1].cells[0].style.color = 'green';
	}
	else if (dryerStatus == "STANDBY") {
		status.rows[1].cells[0].style.color = '#CCCC00';
	}
	else if (dryerStatus == "SHUTDOWN") {
		status.rows[1].cells[0].style.color = 'red';
	}

	status.rows[1].cells[0].innerHTML = dryerStatus;

}
function colorCodeNumber(element, number, highThreshold, lowThreshold) {
	if (number > highThreshold || number < lowThreshold) {
		element.style.color = 'red';
	}
	else {
		element.style.color = 'green';
	}
}
function colorCodeLastRuntime(element, minsThreshold, secsThreshold, mins, secs) {
	if (mins == minsThreshold && secs > secsThreshold) {
		element.style.color = 'red';
	}
	else if (mins > minsThreshold) {
		element.style.color = 'red';
	}
	else {
		element.style.color = 'green';
	}

}

function updateSystem1() {
	var s1 = document.getElementById('compressor_runtime');
	var s1outlet = document.getElementById('sys_1_outlet');
	var s1Comp = document.getElementById('system_1_comp');
	if (p4200.systems[0].status == "ONLINE") {
		s1.rows[3].cells[1].style.color = "green"
	}
	else if (p4200.systems[0].status == "STANDBY") {
		s1.rows[3].cells[1].style.color = "#CCCC00"
	}
	else if (p4200.systems[0].status == "SHUTDOWN") {
		s1.rows[3].cells[1].style.color = "red"
	}
	s1.rows[3].cells[1].innerHTML = p4200.systems[0].status;
	
	if(p4200.sensors.systemOneHumidity.toFixed(2) < 1.1)
	{
		s1outlet.rows[0].cells[1].innerHTML = "0.00 %";
		colorCodeNumber(s1outlet.rows[0].cells[1], p4200.sensors.systemOneHumidity, p4200.highHumidityThreshold, 0);
	}
	
	else if(p4200.sensors.systemOneHumidity.toFixed(2) <= 100 && p4200.sensors.systemOneHumidity.toFixed(2) > 1)
	{
		s1outlet.rows[0].cells[1].innerHTML = p4200.sensors.systemOneHumidity.toFixed(2) + " %";
		colorCodeNumber(s1outlet.rows[0].cells[1], p4200.sensors.systemOneHumidity, p4200.highHumidityThreshold, 0);
	}
	else
	{
		s1outlet.rows[0].cells[1].innerHTML = "DISCONN.";
		s1outlet.rows[0].cells[1].style.color = 'red';
	}

	s1.rows[0].cells[1].innerHTML = p4200.systems[0].compressorLastRunTimeMinutes.pad() + ":" + p4200.systems[0].compressorLastRunTimeSeconds.pad() + " MIN";
	colorCodeLastRuntime(s1.rows[0].cells[1], p4200.highCompressorLastRuntimeThresholdMinutes, p4200.highCompressorLastRuntimeThresholdSeconds, p4200.systems[0].compressorLastRunTimeMinutes, p4200.systems[0].compressorLastRunTimeSeconds);
	
	s1.rows[1].cells[1].innerHTML = p4200.systems[0].dutyCycle + " %";
	colorCodeNumber(s1.rows[1].cells[1], p4200.systems[0].dutyCycle, p4200.highDutyCycleThreshold, 0);

	s1.rows[2].cells[1].innerHTML = p4200.systems[0].compressorTotalRuntimeHours + " HRS";
	colorCodeNumber(s1.rows[2].cells[1], p4200.systems[0].compressorTotalRuntimeHours, p4200.highCompressorTotalRuntimeThreshold, 0);
}

function updatePressures() {
	var outlet = document.getElementById('sys_1_outlet');
	var tank = document.getElementById('tank_pressure');

	outlet.rows[1].cells[1].innerHTML = p4200.sensors.outletPressure.toFixed(2) + " "+pressureUnits
	colorCodeNumber(outlet.rows[1].cells[1], p4200.sensors.outletPressure, p4200.highOutletPressureThreshold, p4200.lowOutletPressureThreshold);
	outlet.rows[2].cells[1].innerHTML = p4200.sensors.flow.toFixed(0) + " "+flowUnits
	colorCodeNumber(outlet.rows[2].cells[1], p4200.sensors.flow, p4200.highFlowThreshold, 0);
	tank.rows[1].cells[0].innerHTML = p4200.sensors.tankPressure.toFixed(2) + " "+pressureUnits

}
function updateAlarmStatus() {
	var alarmStatus = document.getElementById('alarm_status');
	for (var i = 0; i < p4200.alarms.alarms.length; i++) {
		if (p4200.alarms.alarms[i] == true) {
			alarmStatus.rows[1].cells[0].style.color = 'red';
			alarmStatus.rows[1].cells[0].innerHTML = "ALARM";
			document.getElementById('red_dot').style.display = 'block';

			return;
		}
	}
	alarmStatus.rows[1].cells[0].style.color = 'green';
	alarmStatus.rows[1].cells[0].innerHTML = "NO ALARM";
	document.getElementById('red_dot').style.display = 'none';
}
function updateTemps() {
	var temp = document.getElementById('temp');
	
	temp.rows[1].cells[0].innerHTML = p4200.sensors.cabinetTemperature.toFixed(2) + " "+tempUnits;
	colorCodeNumber(temp.rows[1].cells[0], p4200.sensors.cabinetTemperature, p4200.highcabinetTemperatureThreshold, 0);
}
function getP4200() {
	return p4200;
}

function onClearAlarms() {
	if (isUnlocked()) {
		sendMessage(socket,"CLEAR ALARMS")
	}
	
}
function onResetSys1Runtime() {
	if (isUnlocked()) {
	sendMessage(socket,"RESET SYSTEM 1 RUNTIME")
	}
}
function onResetSys2Runtime() {
	if (isUnlocked()) {
	sendMessage(socket,"RESET SYSTEM 2 RUNTIME")
	}
}
