package com.AltecAir.P4200.web.server;


import java.io.IOException;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.format.DateTimeFormatter;

import com.AltecAir.P4200.org.nanohttpd.protocols.http.IHTTPSession;
import com.AltecAir.P4200.org.nanohttpd.protocols.websockets.CloseCode;
import com.AltecAir.P4200.org.nanohttpd.protocols.websockets.NanoWSD;
import com.AltecAir.P4200.org.nanohttpd.protocols.websockets.WebSocket;
import com.AltecAir.P4200.org.nanohttpd.protocols.websockets.WebSocketFrame;

import com.AltecAir.P4200.Screens.SetupScreens.System.FirmwareUpdatingScreen;
import com.AltecAir.P4200.Sensors.EmulatedSensors;
import com.AltecAir.P4200.System.P4200;
import com.AltecAir.P4200.System.SystemRuntime;
import com.AltecAir.P4200.states.InterfaceStateMachine;
import com.AltecAir.Common.util.NetworkTools;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializationFeature;
import com.fasterxml.jackson.datatype.jsr310.JavaTimeModule;

public class P4200WebSocketServer extends NanoWSD
{
	private P4200 p4200;
	private InterfaceStateMachine stateMachine;
	private P4200WebSocket websocket;

	public P4200WebSocketServer(int port, P4200 p4200, InterfaceStateMachine stateMachine)
	{
		super(port);
		this.p4200 = p4200;
		this.stateMachine = stateMachine;
	}

	public InterfaceStateMachine getStateMachine()
	{
		return stateMachine;
	}

	public P4200 getP4200()
	{
		return p4200;
	}

	public void update()
	{
		if (websocket != null && websocket.isOpen())
		{
			websocket.update();
		}

	}

	public void ping()
	{

		if (websocket != null && websocket.isOpen())
		{
			try
			{
				websocket.ping(new byte[]
				{ 0xA });
			}
			catch (IOException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	@Override
	protected WebSocket openWebSocket(IHTTPSession handshake)
	{
		// TODO Auto-generated method stub
		this.websocket = new P4200WebSocket(this, handshake);
		return websocket;
	}

	private static class P4200WebSocket extends WebSocket
	{
		private final P4200WebSocketServer server;
		private ObjectMapper mapper = new ObjectMapper().registerModule(new JavaTimeModule());

		public P4200WebSocket(P4200WebSocketServer server, IHTTPSession handshakeRequest)
		{
			super(handshakeRequest);
			mapper.configure(SerializationFeature.FAIL_ON_EMPTY_BEANS, false);
			this.server = server;

		}

		@Override
		protected void onOpen()
		{
			// TODO Auto-generated method stub

		}

		@Override
		protected void onClose(CloseCode code, String reason, boolean initiatedByRemote)
		{
			// TODO Auto-generated method stub

		}

		private String getCommand(String message)
		{
			if (message.contains(","))
			{
				return message.substring(0, message.indexOf(','));
			}
			else
			{
				return message;
			}

		}

		private static String getData(int index, String message)
		{
			String data = message;
			// strip the command
			data = data.substring(message.indexOf(',') + 1);

			// get rid of all of the data previous to our requested data
			for (int i = 0; i < index; i++)
			{
				data = data.substring(message.indexOf(','));
			}

			if (data.contains(","))
			{
				data = data.substring(0, data.indexOf(','));
			}

			return data;
		}

		@Override
		protected void onMessage(WebSocketFrame message)
		{
			String payload = message.getTextPayload();
			switch (getCommand(payload))
			{
			case "CLEAR ALARMS":
			{
				server.getP4200().getAlarms().clear(server.getP4200().getSnmp());
				break;
			}
			case ("ALARM DELAY ENABLE"):
			{

				String data = getData(0, payload);
				if (data.equals(new String("TRUE")))
				{
					server.p4200.enableAlarmDelay(true);
				}
				else
				{
					server.p4200.enableAlarmDelay(false);
				}
				break;
			}
			case ("SET STARTUP DELAY"):
			{

				String data = getData(0, payload);

				if (data.equals(new String("NONE")))
				{
					server.p4200.setStartupDelay(0);
				}
				else if (data.equals(new String("10 SECONDS")))
				{
					server.p4200.setStartupDelay(10);
				}
				else
				{
					server.p4200.setStartupDelay(Integer.parseInt(data.substring(0, 1)));
				}
				break;
			}
			case ("SET UNITS"):
			{

				String data = getData(0, payload);

				if (data.equals(new String("Imperial")))
				{
					if (server.getP4200().isMetricUnitsEnabled())
					{
						server.p4200.setMetricUnitsEnabled(false);
						server.getP4200().convertThresholdsToImperial();

					}
				}
				else if (data.equals(new String("Metric")))
				{
					if (!server.getP4200().isMetricUnitsEnabled())
					{
						server.p4200.setMetricUnitsEnabled(true);
						server.getP4200().convertThresholdsToMetric();

					}

				}

				break;
			}
			case ("SET DATE"):
			{
				LocalDateTime initialDate = LocalDateTime.now();
				String data = getData(0, payload);
				String[] splitDate = data.split("/");
				int month = Integer.parseInt(splitDate[0]);
				int day = Integer.parseInt(splitDate[1]);
				int year = Integer.parseInt(splitDate[2]);
				StringBuilder sb = new StringBuilder();
				sb.append(String.format("%02d", month) + "/");
				sb.append(String.format("%02d", day) + "/");
				sb.append(String.format("%4d", year));
				DateTimeFormatter fmt = DateTimeFormatter.ofPattern("MM/dd/yyyy");
				LocalDate ld = LocalDate.parse(sb.toString(), fmt);
				LocalDateTime date = LocalDateTime.of(ld, LocalDateTime.now().toLocalTime());

				// set the "Now Time" for the systems so that we don't get
				// errors with compressor runtime if compressors are running
				SystemRuntime.enable(false);
				while(!SystemRuntime.done());
				server.getP4200().getSystems().get(0).setNow(date);
				DateTimeFormatter dateFmt = DateTimeFormatter.ofPattern("yyyyMMdd");
				DateTimeFormatter timeFmt = DateTimeFormatter.ofPattern("HH:mm:ss");
				String dateString = dateFmt.format(date);
				String timeString = timeFmt.format(date);
				server.getP4200().database().insertParameterChange(
						"Date Changed From \"" + fmt.format(initialDate) + "\" to \"" + fmt.format(date) + "\"", date);
				try
				{
					Runtime.getRuntime().exec("sudo date --set +%Y%m%d -s " + dateString);
					Thread.sleep(100);
					Runtime.getRuntime().exec("sudo date --set +%T -s " + timeString);
					Thread.sleep(100);
					// write data to RTC
					Runtime.getRuntime().exec("sudo hwclock -w");
					Thread.sleep(500);
					server.getP4200().restartSystemRuntimeCalculations();
				}
				catch (IOException e)
				{
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				catch (InterruptedException e)
				{
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				SystemRuntime.enable(true);
				break;
			}
			case ("SET TIME"):
			{
				LocalDateTime intialTime = LocalDateTime.now();
				String data = getData(0, payload);
				LocalTime time = LocalTime.parse(data);
				LocalDateTime dateTime = LocalDateTime.now();
				dateTime = dateTime.withHour(time.getHour());
				dateTime = dateTime.withMinute(time.getMinute());
				SystemRuntime.enable(false);
				while(!SystemRuntime.done());
				server.getP4200().getSystems().get(0).setNow(dateTime);
				DateTimeFormatter timeFmt = DateTimeFormatter.ofPattern("HH:mm:ss");
				String timeString = timeFmt.format(dateTime);
				DateTimeFormatter dispTimeFmt = DateTimeFormatter.ofPattern("HH:mm");
				server.getP4200().database()
						.insertParameterChange("Time Changed From \"" + dispTimeFmt.format(intialTime.toLocalTime())
								+ "\" to \"" + dispTimeFmt.format(dateTime.toLocalTime()) + "\"", dateTime);
				try
				{

					Runtime.getRuntime().exec("sudo date --set +%T -s " + timeString);
					Thread.sleep(100);
					// write data to RTC
					Runtime.getRuntime().exec("sudo hwclock -w");
					Thread.sleep(500);
					server.getP4200().restartSystemRuntimeCalculations();
				}
				catch (IOException e)
				{
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				catch (InterruptedException e)
				{
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				SystemRuntime.enable(true);
				break;
			}
			case ("SET DEV ID"):
			{
				String data = getData(0, payload);
				server.getP4200().setDeviceID(data);
				break;
			}
			case ("SET MODEL NUMBER"):
			{
				String data = getData(0, payload);
				server.getP4200().setModelNumber(data);
				break;
			}
			case ("SET SERIAL NUMBER"):
			{
				String data = getData(0, payload);
				server.getP4200().setSerialNumber(data);
				;
				break;
			}
			case ("SET BRANDING"):
			{
					String data = getData(0, payload);
					server.getP4200().setBranding(data);
					break;
			}
			case ("SET MASTER PASSWORD"):
			{
				String data = getData(0, payload);
				server.getP4200().setMasterKeyword(Integer.parseInt(data));
				;
				break;
			}
			case ("SET HIGH OUTLET PRESSURE THRESHOLD"):
			{
				String data = getData(0, payload);
				server.p4200.setHighOutletPressureThreshold((Float.parseFloat(data)));
				break;
			}
			case ("SET LOW OUTLET PRESSURE THRESHOLD"):
			{
				String data = getData(0, payload);
				server.p4200.setLowOutletPressureThreshold((Float.parseFloat(data)));
				break;
			}
			case ("SET HIGH FLOW THRESHOLD"):
			{
				String data = getData(0, payload);
				server.p4200.setHighFlowThreshold(Float.parseFloat(data));
				break;
			}
			case ("SET HIGH HUMIDITY THRESHOLD"):
			{
				String data = getData(0, payload);
				server.p4200.setHighHumidityThreshold(Float.parseFloat(data));
				break;
			}
			case ("SET HIGH DUTY THRESHOLD"):
			{
				String data = getData(0, payload);
				server.p4200.setHighDutyThreshold(Integer.parseInt(data));
				break;
			}
			case ("SET HIGH COMP RUNTIME THRESHOLD"):
			{
				String data = getData(0, payload);
				int minutes = Integer.parseInt(data.substring(0, data.indexOf(':')));

				int seconds = Integer.parseInt(data.substring(data.indexOf(':') + 1));

				server.getP4200().setHighCompressorLastRuntimeThresholdMinutes(minutes);
				server.getP4200().setHighCompressorLastRuntimeThresholdSeconds(seconds);
				break;
			}
			case ("SET IP"):
			{
				String data = getData(0, payload);

				server.getP4200().getNetwork().changeIPAddress(NetworkTools.stringToIP(data));
				break;
			}
			case ("SET SUBNET"):
			{
				String data = getData(0, payload);

				server.getP4200().getNetwork()
						.setSubnetMask((byte) (NetworkTools.convertNetmaskToCIDR(NetworkTools.stringToIP(data))));
				break;
			}
			case ("SET GATEWAY"):
			{
				String data = getData(0, payload);

				server.getP4200().getNetwork().setDefaultGateway(NetworkTools.stringToIP(data));
				break;
			}
			case ("SET SNMP"):
			{
				String data = getData(0, payload);

				server.getP4200().getNetwork().changeSNMPTrap(NetworkTools.stringToIP(data));
				break;
			}
			case ("RESET TO FACTORY DEFAULT"):
			{

				server.getP4200().factoryReset();
				break;
			}
			case ("CHANGE KEYWORD"):
			{
				String data = getData(0, payload);
				server.getP4200().setKeyword(Integer.parseInt(data));
				break;
			}
			case ("ENABLE EMULATION"):
			{
				server.getP4200().enableEmulation();
				break;
			}
			case ("ENABLE EXTENDED ALARM DELAY"):
			{
				server.getP4200().enableExtendedAlarmDelay(true);
				break;
			}
			case ("DISABLE EXTENDED ALARM DELAY"):
			{
				server.getP4200().enableExtendedAlarmDelay(false);
				break;
			}
			case ("CLEAR EVENT LOG"):
			{
				server.getP4200().database().clearTable();
				break;
			}
			case ("DISABLE EMULATION"):
			{
				server.getP4200().disableEmulation();
				break;
			}
			case "RESET SYSTEM 1 RUNTIME":
			{
				server.getP4200().resetRuntime();
				break;
			}
			case ("SET SYSTEM ONE HUMIDITY"):
			{
				String data = getData(0, payload);
				EmulatedSensors sensors = (EmulatedSensors) server.getP4200().getSensors();
				sensors.setSystemOneHumidity((Float.parseFloat(data)));
				break;
			}

			case ("SET SYSTEM TWO HUMIDITY"):
			{
				String data = getData(0, payload);
				EmulatedSensors sensors = (EmulatedSensors) server.getP4200().getSensors();
				sensors.setSystemTwoHumidity((Float.parseFloat(data)));
				break;
			}
			case ("SET TANK PRESSURE"):
			{
				String data = getData(0, payload);
				EmulatedSensors sensors = (EmulatedSensors) server.getP4200().getSensors();
				sensors.setTankPressure((Float.parseFloat(data)));
				break;
			}
			case ("SET OUTLET PRESSURE"):
			{
				String data = getData(0, payload);
				EmulatedSensors sensors = (EmulatedSensors) server.getP4200().getSensors();
				sensors.setOutletPressure((Float.parseFloat(data)));
				break;
			}
			case ("SET FLOW"):
			{
				String data = getData(0, payload);
				EmulatedSensors sensors = (EmulatedSensors) server.getP4200().getSensors();
				sensors.setFlow((Float.parseFloat(data)));
				break;
			}
			case ("SET COMPRESSOR TOTAL RUNTIME"):
			{
				String data = getData(0, payload);
				server.getP4200().setCompressorHours(Long.parseLong(data));
				break;
			}
			case ("SET COMPRESSOR TEMP"):
			{
				String data = getData(0, payload);
				EmulatedSensors sensors = (EmulatedSensors) server.getP4200().getSensors();
				sensors.setCompressorTemperature((Float.parseFloat(data)));
				break;
			}
			case ("SET CABINET TEMP"):
			{
				String data = getData(0, payload);
				EmulatedSensors sensors = (EmulatedSensors) server.getP4200().getSensors();
				sensors.setCabinTemperature((Float.parseFloat(data)));
				break;
			}
			case ("FIRMWARE UPDATING"):
			{
				server.getStateMachine().setTranistionEnabled(false);
				try
				{
					Thread.sleep(1000);
				} catch (InterruptedException e)
				{
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				server.getStateMachine().setScreen(new FirmwareUpdatingScreen());
				
				server.getStateMachine().displayScreen();
				
				
				server.getP4200().database().insertInfo("Firmware Updated", LocalDateTime.now());
				break;
			}
			}

		}

		@Override
		protected void onPong(WebSocketFrame pong)
		{

		}

		@Override
		protected void onException(IOException exception)
		{
			// TODO Auto-generated method stub

		}

		public void update()
		{
			try
			{
				String json = mapper.writeValueAsString(server.getP4200());
				if (this.isOpen())
				{
					send(json);
				}

			}
			catch (JsonProcessingException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			catch (IOException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

}
